#
# RecurringEvent/tests/testSetup.py
#
# Test RecurringEvent initialisation and set-up
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.RecurringEvent.tests import retc
from Products.RecurringEvent.config import RECUE_MODES, RECUE_OFFSETS
from Products.RecurringEvent.config import RM_ONE_TIME, RM_UNTIL, RM_FOREVER
from Products.RecurringEvent.config import RO_DAILY, RO_WEEKLY, RO_MONTHLY, RO_YEARLY, FROZEN_DATE_IN_FUTURE


from DateTime import DateTime

class TestInstallation(retc.RecurringEventTestCase):
    """Ensure product is properly installed"""

    def afterSetUp(self):
        self.css        = self.portal.portal_css
        self.kupu       = self.portal.kupu_library_tool
        self.skins      = self.portal.portal_skins
        self.types      = self.portal.portal_types
        self.factory    = self.portal.portal_factory
        self.workflow   = self.portal.portal_workflow
        self.properties = self.portal.portal_properties

        self.metaTypes = ('RecurringEvent',)

    #def testSkinLayersInstalled(self):
    #   self.failUnless('RecurringEvent' in self.skins.objectIds())
    #    self.failUnless('attachment_widgets' in self.skins.objectIds())
        
    #def testTypesInstalled(self):
    #    for t in self.metaTypes:
    #        self.failUnless(t in self.types.objectIds())

    #def testPortalFactorySetup(self):
    #    self.failUnless('RichDocument' in self.factory.getFactoryTypes())

    #def testTypesNotSearched(self):
    #    types_notes_searched = self.properties.site_properties.getProperty('types_not_searched')
    #    self.failUnless('FileAttachment' in types_not_searched)
    #    self.failUnless('ImageAttachment' in types_not_searched)

    #def testCssInstalled(self):
    #    self.failUnless('richdocument.css' in self.css.getResourceIds())
    
    #def testDefaultPageType(self):
    #    default_page_types = self.properties.site_properties.getProperty('default_page_types')
    #    self.failUnless('RichDocument' in default_page_types)
    
    #def testParentMetaTypesNotToQuery(self):
    #    parentMetaTypesNotToQuery = self.properties.navtree_properties.getProperty('parentMetaTypesNotToQuery')
    #    self.failUnless('RichDocument' in parentMetaTypesNotToQuery)

    #def testTypesNotSearched(self):
    #    types_not_searched = self.properties.site_properties.getProperty('types_not_searched')
    #    self.failUnless('FileAttachment' in types_not_searched)
    #    self.failUnless('ImageAttachment' in types_not_searched)

    #def testAttachmentsHaveNoWorkflow(self):
    #    self.assertEqual(self.workflow.getChainForPortalType('FileAttachment'), ())
    #    self.assertEqual(self.workflow.getChainForPortalType('ImageAttachment'), ())
    
    #def testKupuResources(self):
    #    linkable = self.kupu.getPortalTypesForResourceType('linkable')
    #    mediaobject = self.kupu.getPortalTypesForResourceType('mediaobject')
    #    self.failUnless('RichDocument' in linkable)
    #    self.failUnless('FileAttachment' in linkable)
    #    self.failUnless('ImageAttachment' in linkable)
    #    self.failUnless('ImageAttachment' in mediaobject)

class TestContentCreation(retc.RecurringEventTestCase):
    """Ensure content types can be created and edited"""

    def afterSetUp(self):
        self.folder.invokeFactory('RecurringEvent', 're1')
        self.re1 = getattr(self.folder, 're1')

    def testCreateRecurringEvent(self):
        self.failUnless('re1' in self.folder.objectIds())

    def testEditRecurringEdit(self):
        self.re1.setTitle('A title')
        self.re1.setDescription('A description')
        self.re1.setText('<p>Body text</p>')
        sd = DateTime('2006/01/31 12:35')
        self.re1.setStartDate(sd)
        ed = DateTime('2006/01/31 12:45')	  
        self.re1.setEndDate(ed)
        rud = DateTime('2007/02/27 12:30')
        self.re1.setRecueUntilDate(rud)
 
        self.re1.setRecueMode(RM_ONE_TIME)
        self.re1.setRecueOffset(RO_DAILY)

        self.assertEqual(self.re1.Title(), 'A title')
        self.assertEqual(self.re1.Description(), 'A description')
        self.assertEqual(self.re1.getText(), '<p>Body text</p>')
        self.failUnless(self.re1.startDate.equalTo(sd))
        self.failUnless(self.re1.endDate.equalTo(ed))
        self.failUnless(self.re1.getRecueUntilDate().equalTo(rud))
        self.assertEqual(self.re1.getRecueMode(), RM_ONE_TIME)
        self.assertEqual(self.re1.getRecueOffset(), RO_DAILY)
     
    def testDefaultValues(self):
        self.assertEqual(self.re1.defaultRecueMode(), RECUE_MODES.items()[0])
        self.assertEqual(self.re1.defaultRecueOffset(), RECUE_OFFSETS.items()[0])

    def testNumberOfDaysInMonth(self):
        dt = DateTime('2000/01/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/02/01')
        self.assertEqual(29, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2001/02/01')
        self.assertEqual(28, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/03/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/04/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/05/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/06/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/07/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/08/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/09/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/10/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/11/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))
        
        dt = DateTime('2000/12/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))
        
        
    def testNumberOfDaysInYear(self):
        dt = DateTime('2000/12/01')
        self.assertEqual(366, self.re1.numberOfDaysInYear(dt))

        dt = DateTime('2001/12/01')
        self.assertEqual(365, self.re1.numberOfDaysInYear(dt))
        
    def testShiftDateTimeByOffset(self):
        self.re1.setRecueMode('until')
        self.re1.setRecueOffset('daily')
        
        #standardYear version
        dt = DateTime('2001/12/01')
        self.failUnless(DateTime('2001/12/02').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        #leapYear version
        dt = DateTime('2000/02/28')
        self.failUnless(DateTime('2000/02/29').equalTo(self.re1.shiftDateTimeByOffset(dt)))

        dt = DateTime('2000/12/01')
        self.re1.setRecueOffset('weekly')
        self.failUnless(DateTime('2000/12/08').equalTo(self.re1.shiftDateTimeByOffset(dt)))

		#leapYear version
        dt = DateTime('2000/02/01')
        self.re1.setRecueOffset('monthly')
        self.failUnless(DateTime('2000/03/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))

		#standardYear version
        dt = DateTime('2001/02/01')
        self.re1.setRecueOffset('monthly')
        self.failUnless(DateTime('2001/03/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))
		
		#year boundary
        self.re1.setRecueOffset('yearly')
		# additional leapYear testing
        dt = DateTime('2000/12/31')
        self.failUnless(dt.dayOfYear() == 366)
        
        dt = DateTime('1999/02/28')
        self.failUnless(DateTime('2000/02/28').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        dt = DateTime('1999/03/01')
        self.failUnless(DateTime('2000/03/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        dt = DateTime('2000/02/28')
        self.failUnless(DateTime('2001/02/28').equalTo(self.re1.shiftDateTimeByOffset(dt)))
		
        dt = DateTime('2000/03/01')
        self.failUnless(DateTime('2001/03/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))

        dt = DateTime('2000/12/01')
        #leapYear version
        self.failUnless(dt.isLeapYear())
        self.failUnless(DateTime('2001/12/02').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        #standardYear version
        dt = DateTime('2001/12/01')
        self.failUnless(not dt.isLeapYear())
        self.failUnless(DateTime('2002/12/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
    def testRecueDays(self):
        self.re1.setStartDate(DateTime('2001/01/01 12:30'))
        self.re1.setRecueUntilDate(DateTime('2001/01/31 5:20'))
       
        days = self.re1.recueDays()
       
        #self.failUnless(len(days) == 31)       
        


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestContentCreation))
    return suite
